import _, { round } from 'lodash';
import react, { useState, useEffect, useMemo } from 'react';

import Tile from './tile';

import '../theme/map.css';

export const tileSize = 256;

// cf https://wiki.openstreetmap.org/wiki/Slippy_map_tilenames#ECMAScript_(JavaScript/ActionScript,_etc.)

const lon2tile = (lon: number, zoom: number) => {
  const real = ((lon + 180) / 360) * Math.pow(2, zoom);
  const floor = Math.floor(real);
  return [floor, real - floor];
};
const lat2tile = (lat: number, zoom: number) => {
  const real =
    ((1 -
      Math.log(
        Math.tan((lat * Math.PI) / 180) + 1 / Math.cos((lat * Math.PI) / 180)
      ) /
        Math.PI) /
      2) *
    Math.pow(2, zoom);
  const floor = Math.floor(real);
  return [floor, real - floor];
};

const Map: react.FC = () => {
  const initialCenter: [number, number] = [43.57029965, 3.94242897];
  const initialZoom: number = 17;
  const [center, setCenter] = useState(initialCenter);
  const [zoom, setZoom] = useState(initialZoom);

  const [dimensions, setDimensions] = useState({
    height: window.innerHeight,
    width: window.innerWidth,
  });

  const handleResize = () => {
    setDimensions({
      height: window.innerHeight,
      width: window.innerWidth,
    });
  };
  const debouncedEventHandler = useMemo(
    () => _.debounce(handleResize, 300)
  , [dimensions, zoom, center]);

  useEffect(() => {
    window.addEventListener('resize', debouncedEventHandler);
  });

  const nbTilesY = _.ceil(dimensions.height / tileSize) + 2;
  const nbTilesX = _.ceil(dimensions.width / tileSize) + 2;
  const [tileCenterY, reminderY] = lat2tile(center[0], zoom);
  const [tileCenterX, reminderX] = lon2tile(center[1], zoom);
  const firstTileY = tileCenterY - _.ceil(nbTilesY / 2);
  const firstTileX = tileCenterX - _.ceil(nbTilesX / 2);
  const locationY = ((tileCenterY + reminderY - firstTileY ) * tileSize) ;
  const locationX = ((tileCenterX + reminderX - firstTileX ) * tileSize) ;
  const targetLocationY = dimensions.height / 2;
  const targetLocationX = dimensions.width / 2;
  const deltaY = - targetLocationY + locationY;
  const deltaX = - targetLocationX + locationX;

  return (
    <div className='tiles'>
      {_.range(nbTilesY).map((iy) => (
        <div key={'y' + iy} className='tilesRow'>
          {_.range(nbTilesX).map((ix) => (
            <Tile
              key={'x' + ix}
              iy={iy}
              ix={ix}
              x={firstTileX + ix}
              y={firstTileY + iy}
              deltaX={deltaX}
              deltaY={deltaY}
              zoom={zoom}
            />
          ))}
        </div>
      ))}
    </div>
  );
};

export default Map;
